/*
 * Seed enCoding Automaton (sca)
 *
 * fast encoding of seeds using a shift or approach
 * (not in the paper, due to space limitations ... 
 *  but not very difficult to understand )
 *
 */

#ifndef __SCA_H__
#define __SCA_H__

//STL
#include <vector>
#include <list>

//STD
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>
#include <string>
using namespace std;

//STR
#include "macro.h"
#include "iodna.h"
 

#define MAXSPAN 512

string reverseMotif(string& m);

class sca /* seed enCoding automaton */
{
 public:
  
  sca(string& motif,bool reverse);
  
  /* Handling methods */
         int init(seqdna& seqdna,int pos=0);
  inline int hasNext() { return _seqdnapos <= (int)_seqdna.dnasize() - _motif_span; };
    SEEDCODE next();

#ifdef MASKED
  inline int isMasked(){ return _masked; };
#endif

  inline int getDnaPos()    {return _seqdnapos ;};/* 0 .. n */
  inline int getBytePos()   {return _seqbytepos;};/* 0 .. n */
  inline int getBitWeight() {return _motif_bitweight;}; // give the seed bitweight */
  inline int getSpan()      {return _motif_span;};
  
  
      
  /* IO methods */
  friend ostream& operator<<(ostream& os, sca& sca);

 protected:
  
  bool             _reversed;
  /* black box tables (used in the shift-or encoding) */
  int              _auto_shift[MAXSPAN];
  unsigned char    _auto_or[MAXSPAN][256];
  SEEDCODE         _auto_states[MAXSPAN];
  int              _auto_nbboxes; //size used in the 3 previous tables (replace MAXSPAN) 

  /* black box positions (dont play with those variable during the encoding part) */ 
  int              _auto_fu;  //first unfinished (first seed code unfinished)
  int              _auto_pos; //

#ifdef MASKED
  int              _masked;
  vector<int>::iterator  _it_m;
#endif

  /* sequence */
  int              _seqbytepos; //first unread byte 
  int              _seqdnapos ; //first unread nucleotide (used in hasNext)
  seqdna           _seqdna;     //sequence
  
  /* motif */
  string           _motif;
  int              _motif_span;
  int              _motif_bitweight;
};


#endif

